/* This file was automatically generated on Fri 25 May 2018 08:06:10 BST from
 * source: /home/hgv27681/R3.14.12.3/support/pmac/etc/makeIocs/lab.xml
 *
 * *** Please do not edit this file: edit the source file instead. ***
 *  */
#include "epicsExit.h"
#include "epicsThread.h"
#include "iocsh.h"

int main(int argc, char *argv[])
{
    if(argc>=2) {
        iocsh(argv[1]);
        epicsThreadSleep(.2);
    }
    iocsh(NULL);
    epicsExit(0);
    return 0;
}

